import logging

from portal.database import Project
from portal.database.models import Client, Course
from portal.facade.general_facade import GeneralCRUDFacade
from portal.service import errors, filters
from portal.service.projects import ProjectService

log = logging.getLogger(__name__)


class ProjectsFacade(GeneralCRUDFacade):
    def __init__(self):
        super().__init__(ProjectService, 'Project')

    def find_all(self, course: Course, *args, **kwargs):
        course_perm = self.permissions(course=course)
        if course_perm.check.view_course_full():
            return self._service.find_all(*args, course=course, **kwargs)
        elif course_perm.check.view_course_limited():
            return filters.filter_projects_from_course(course=course, user=self.client)
        raise errors.ForbiddenError(self.client)

    def update_project_config(self, project: Project, **params):
        """Updates the project configuration for the project
        Args:
            project: Project Instance
            **params:
        """
        log.info(f"[UPDATE] Configuration for project {project.log_name}"
                 f" by {self.client_name}: {params}.")
        result = self._service(project).update_project_config(params)
        return result

    def update_project_test_files(self, project: Project):
        """Update the project test files

        Downloads the new version from the gitlab
        Args:
            project: project instance

        """
        # TODO invoke the async task
        result = self._service(project).update_project_test_files()
        return result

    def find_project_submissions(self, project: Project, user_id: str):
        """Finds all project submissions. If a user id is specified, returns only submissions created
        by that user.

        Args:
            project(Project): Project
            user_id(str): User id (optional)

        Returns(List[Submission]): A list of all submissions in a project if no user_id is specified
        Otherwise a list of submissions created in the project by the specified user.

        """
        result = self._service(project).find_project_submissions(user_id)
        return result

    def check_submission_create(self, project: Project, client: Client):
        """Check whether client can create a submission
        Args:
            project: project instance
            client: client instance
        """
        client = client or self.client
        result = self._service(project).check_submission_create(client)
        return result

    def get_test_files(self, project: Project):
        """Gets the test files for the project
        Args:
            project: project instance
        """
        log.debug(f"Getting test files for the project: {project.log_name}"
                  f" by {self.client.log_name}")
        service = self.services.storage(project=project)
        storage_entity = service.get_test_files_entity_from_storage()
        path = self._request.args.get('path')
        return service.send_file_or_zip(storage_entity=storage_entity, path=path)

    def copy_project(self, project: Project, target_course: Course):
        """Copies the project
        Args:
            project(Project): Project instance
            target_course(Course): Course instance
        """
        log.info(f"[COPY] Project from {project.log_name} to {target_course.log_name} by"
                 f"{self.client_name}")
        result = self._service(project).copy_project(target_course)
        return result
