import logging
from typing import List, Set

from portal.database import Course, User
from portal.service.rest import RestService
from portal.tools.is_api_adapter import IsApiFactory

log = logging.getLogger(__name__)


def _users_to_ucos_set(users: List[User]) -> Set[int]:
    return set(user.uco for user in users)


class IsApiService:
    def __init__(self, rest_service):
        self._rest_service: RestService = rest_service

    @property
    def is_factory(self) -> IsApiFactory:
        from portal import is_api_factory
        return is_api_factory

    def import_students(self, course: Course, role_name: str = 'student'):
        is_api = self.is_factory.get_instance(course=course)
        students = is_api.course_students
        ucos = [student.uco for student in students.students]
        self._import_users(course, ucos, role_name)

    def import_teachers(self, course: Course, role_name: str = 'teacher'):
        is_api = self.is_factory.get_instance(course=course)
        teachers = is_api.course_teachers
        ucos = [teacher.uco for teacher in teachers]
        self._import_users(course, ucos, role_name)

    def sync_seminaries(self, course: Course):
        is_api = self.is_factory.get_instance(course=course)
        seminaries = is_api.course_seminars_list
        for seminary in seminaries:
            self._sync_seminary(is_api, course, seminary)

    def _import_users(self, course, ucos: List[int], role_name):
        log.info(f"Importing users to course {course.log_name} with role {role_name}: {ucos}")
        role = self._rest_service.find.role(course=course, identifier=role_name)
        for uco in ucos:
            user = User.query.filter(User.uco == uco)
            if not user:
                log.info(f"User with uco: {uco} does not exists - SKIPPING.")
                return
            self._rest_service.roles(role).add_client(user)

    def _sync_seminary(self, is_api, course, seminary):
        group = self._get_or_create_group(course, seminary)
        s_uco = self._extract_ucos_students(is_api, group, seminary)
        t_uco = self._extract_ucos_teachers(is_api, group, seminary)
        current_is = set(*s_uco, *t_uco)
        current_portal = _users_to_ucos_set(group.users)
        remove = current_portal - current_is
        add = current_is - current_portal
        log.info(f"[SYNC] Seminary SYNC - remove: {remove}, add: {add}")
        self._mod_person_in_group(group, add, 'add_user')
        self._mod_person_in_group(group, remove, 'add_user')

    def _get_or_create_group(self, course, seminary):
        group = self._rest_service.find.group(course, seminary.label)
        if not group:
            params = dict(codename=seminary.label, name=f"Seminary {seminary.label}")
            group = self._rest_service.groups.create(course, **params)
        return group

    def _extract_ucos_students(self, is_api, group, seminary):
        items = is_api.is_client.seminar_list_students(seminars=[seminary.label])
        return [item.uco for item in items.seminars[0].students]

    def _add_person_to_group(self, ucos, group):
        for uco in ucos:
            user = User.query.filter(User.uco == uco)
            if not user:
                log.info(f"User with uco: {uco} does not exists - SKIPPING.")
                return
            self._rest_service.groups(group).add_user(user)

    def _extract_ucos_teachers(self, is_api, group, seminary):
        items = is_api.is_client.seminar_list_teachers(seminars=[seminary.label])
        return [item.uco for item in items.seminars[0].teachers]
